/*
 * WTP initiator header
 *
 * Aarno Syvnen for Wapit Ltd
 */

#ifndef WTP_INIT_H
#define WTP_INIT_H

#include "gwlib/gwlib.h"
#include "wap_addr.h"
#include "wap_events.h"
#include "timers.h"

typedef struct sar_info_t {
    int sar_psn;
    Octstr *sar_data;
} sar_info_t;


/*
 * Structure to keep SAR data during transmission
 */
typedef struct WTPSARData {
    int nsegm;  /* number of the last segment, i.e. total number - 1 */
    int csegm;  /* last segment confirmed by recipient */
    int lsegm;  /* last sent segment */
    int tr;		/* if current psn is gtr or ttr */
    Octstr *data;
} WTPSARData;


#define	SAR_SEGM_SIZE 1500
#define	SAR_GROUP_LEN 3
 
 
/*
 * Initiator machine states and initiator WTP machine.
 * See included file for comments. Note that we must define macro
 * ROW to produce an empty string.
 */
enum init_states {
    #define INIT_STATE_NAME(state) state,
    #define ROW(state, event, condition, action, next_state)
    #include "wtp_init_states.def"
    init_states_count
};

typedef enum init_states init_states;

/*
 * See included file for comments. We define one macro for 
 * every separate type.
 */
typedef struct WTPInitMachine {
    unsigned long mid; 
    #define INTEGER(name) int name; 
    #define EVENT(name) WAPEvent *name;
    #define TIMER(name) Timer *name; 
    #define ADDRTUPLE(name) WAPAddrTuple *name;
    #define ENUM(name) init_states name;
    #define LIST(name) List *name;
    #define SARDATA(name) WTPSARData *name;
    #define MACHINE(field) field
    #include "wtp_init_machine.def"
} WTPInitMachine;

#endif
